# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np
import pandas as pd

def rca(tbl, populations=None):
  
  # fill missing values with zeros
  tbl = tbl.fillna(0)

  # get sum over columns
  col_sums = tbl.sum(axis=1)

  # we now need to transpose or "reshape" this array so that
  # it is in the form of one long column
  col_sums = col_sums.values.reshape((len(col_sums), 1))

  # create the numerator matrix for the final RCA calculation by
  # dividing each value by its row's sum
  rca_numerator = np.divide(tbl, col_sums)

  # get the sum over all the rows
  row_sums = tbl.sum(axis=0)

  # if populations is set create the denominator based on that for POP RCA
  if populations.__class__ == pd.DataFrame or populations.__class__ == pd.Series:

    # create the denominator matrix for the final RCA calculation
    # by dividing the industry sums by a single value (the matrix total sum)
    rca_denominator = populations / float(populations.sum())
    
    # rca_denominator = rca_denominator.reshape((len(rca_denominator), 1))
    # print rca_numerator.shape
    
    # rca_denominator = pd.DataFrame(rca_denominator, columns=[rca_numerator.columns[0]])
    # rca_denominator = rca_denominator.reindex(index=rca_numerator.index)
    # rca_denominator = rca_denominator.reindex(columns=rca_numerator.columns, method="ffill")
    # print rca_decnominator.ix["ac"]

    # lastly we get the RCAs by dividing the numerator matrix by denominator
    rcas = rca_numerator.T / rca_denominator
    rcas = rcas.T
    

  else:
    # get total of all the values in the matrix
    total_sum = tbl.sum().sum()

    # create the denominator matrix for the final RCA calculation
    # by dividing the industry sums by a single value (the matrix total sum)
    rca_denominator = row_sums / total_sum

    # lastly we get the RCAs by dividing the numerator matrix by denominator
    rcas = rca_numerator / rca_denominator
    

  # rcas[rcas >= 1] = 1
  # rcas[rcas < 1] = 0
  
  return rcas



# -*- coding: utf-8 -*-
"""
    Product Space Calculations Library
    ~~~~~~~~~~~~~~

    The ps_calcs library is a helper library for running product space
    calculations in python with PANDAS and NumPy.

    :copyright: (c) 2018 by Alexander Simoes.
    :license: MIT, see LICENSE for more details.
"""
__version__ = '2.0.0'
name = "ps_calcs"

from .complexity import complexity
from .rca import rca
from .density import density
from .distance import distance
from .proximity import proximity
from .opportunity_gain import opportunity_gain
from .mhat import mhat


# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np

def proximity(rcas):
  
  # transpose the matrix so that it is now industries as rows and munics as
  # columns
  rcas_t = rcas.T.fillna(0)

  # Matrix multiplication on M_mi matrix and transposed version,
  # number of products = number of rows and vice versa on transposed
  # version, thus the shape of this result will be length of products by
  # by the length of products (symetric)
  numerator_intersection = rcas_t.dot(rcas_t.T)

  # kp0 is a vector of the number of munics with RCA in the given product
  kp0 = rcas.sum(axis=0)
  kp0 = kp0.values.reshape((1, len(kp0)))

  # transpose this to get the unions
  kp0_trans = kp0.T
  
  # multiply these two vectors, take the squre root
  # and then we have the denominator
  # denominator_union = kp0_trans.dot(kp0)
  denominator_union = kp0_trans.dot(kp0)

  # get square root for geometric mean
  denominator_union_sqrt = np.power(denominator_union, .5)

  # to get the proximities it is now a simple division of the untion sqrt
  # with the numerator intersections
  phi = np.divide(numerator_intersection, denominator_union_sqrt)
  
  return phi



# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np
import pandas as pd

def density(rcas, proximities):

  # Get numerator by matrix multiplication of proximities with M_im
  density_numerator = rcas.dot(proximities)

  # Get denominator by multiplying proximities by all ones vector thus
  # getting the sum of all proximities
  # rcas_ones = pd.DataFrame(np.ones_like(rcas))
  rcas_ones = rcas * 0
  rcas_ones = rcas_ones + 1
  # print rcas_ones.shape, proximities.shape 
  density_denominator = rcas_ones.dot(proximities)
  
  # We now have our densities matrix by dividing numerator by denomiator
  densities = density_numerator / density_denominator

  return densities


# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np

def complexity(rcas, drop=True):
  
  rcas_clone = rcas.copy()
  
  # drop columns / rows only if completely nan
  rcas_clone = rcas_clone.dropna(how="all")
  rcas_clone = rcas_clone.dropna(how="all", axis=1)
  
  if rcas_clone.shape != rcas.shape:
    print("[Warning] RCAs contain columns or rows that are entirely comprised of NaN values.")
    if drop:
      rcas = rcas_clone
  
  kp = rcas.sum(axis=0)
  kc = rcas.sum(axis=1)
  kp0 = kp.copy()
  kc0 = kc.copy()

  for i in range(1, 20):
    kc_temp = kc.copy()
    kp_temp = kp.copy()
    kp = rcas.T.dot(kc_temp) / kp0
    if i < 19:
      kc = rcas.dot(kp_temp) / kc0
  
  geo_complexity = (kc - kc.mean()) / kc.std()
  prod_complexity = (kp - kp.mean()) / kp.std()

  return geo_complexity, prod_complexity



# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np
try:
    import statsmodels.api as sm
except ImportError as err:
    print("Warning: failed to import statsmodels module. Unable to " \
                        "run MHAT calculations. ({})".format(err))
import pandas as pd

def mhat(mcp, right_side, regression_type):
    
    # prepare data by unravelling matricies to long lists
    mcp_list = mcp.values.ravel()

    # run regression
    if regression_type == "ols":
        model = sm.OLS(mcp_list, right_side)
        results = model.fit()
    else:
        # print mcp_list[:10]
        # sys.exit()
        
        model = sm.Logit(mcp_list, right_side)
        # model = sm.Logit([1, 1, 1, 0, 0, 0, 1], [1, 0, 0, 0, 1, 1, 1])
        results = model.fit()
        sys.exit()
    
    # print logit_results.__doc__
    # print logit_results.params
    # print results.summary()

    prediction = results.predict(right_side)

    # repackage so it is same shape as mcp
    prediction = prediction.reshape((len(mcp.index), len(mcp.columns)))
    prediction = pd.DataFrame(prediction, index=mcp.index, columns=mcp.columns)

    return prediction



# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np
import pandas as pd
from statsmodels.stats.proportion import proportion_confint
# documentation - http://statsmodels.sourceforge.net/devel/generated/statsmodels.stats.proportion.proportion_confint.html

'''Expects a pivoted DataFrame (tbl). Calculates min and max RCA w/ 95%
    confidence returned as 2 separate DataFrames. Uses Fiellers theorum for
    combining 2 confidence intervals - one for the numerator and another for
    the denominator https://en.wikipedia.org/wiki/Fieller's_theorem.'''
def rca_w_confidence(tbl):
    
    '''Wrapper around confidence interval function to be called on a
        pandas dataframe via .apply(), must return a single series.'''
    def conf_int(series, nobs=None, get_max=False):
        nobs = series.sum() or nobs
        get_max = 1 if get_max else 0
        return proportion_confint(series, nobs, method='wilson')[get_max]
    
    '''Get the min and max confidence interval for numerator'''
    p1hat_min = tbl.apply(conf_int, axis=1)
    p1hat_max = tbl.apply(conf_int, axis=1, args=(None, True))
    
    '''Total value in matrix'''
    grand_total = tbl.sum().sum()
    
    '''Get the min and max confidence interval for denominator'''
    (p2hat_min, p2hat_max) = proportion_confint(ybo_pivot.sum(), grand_total, method='wilson')
    
    '''Project denominator conf intervals to same dimensions as tbl'''
    p2hat_min = pd.DataFrame([p2hat_min]*len(tbl.index))
    p2hat_min.index = tbl.index
    p2hat_max = pd.DataFrame([p2hat_max]*len(tbl.index))
    p2hat_max.index = tbl.index
    
    '''Using Fieller's theorem to combine 2 conf intervals'''
    rsa_min_all_new = p1hat_min / p2hat_max
    rsa_max_all_new = p1hat_max / p2hat_min
    
    return rsa_min_all_new, rsa_max_all_new


"""
ps_calcs
======

The ps_calcs library helps run calculations for creating a product space as
well as other related calculations.

Resources
---------

* `Issue Tracker <https://github.com/alexandersimoes/ps_calcs/issues>`_
* `Source <https://github.com/alexandersimoes/ps_calcs>`_
* `About Product Space <http://en.wikipedia.org/wiki/The_Product_Space>`_

"""

from setuptools import setup
import os

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='ps_calcs',
    version='2.0.0',
    url='https://github.com/alexandersimoes/ps_calcs',
    license='MIT',
    author='Alexander Simoes',
    author_email='simoes@mit.edu',
    description='Product Space calculations.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=[
        'ps_calcs'
    ],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    classifiers=[
        "Programming Language :: Python :: 3",
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Scientific/Engineering :: Mathematics'
    ]
)



# -*- coding: utf-8 -*-

''' Import statements '''
import sys
import numpy as np
import pandas as pd
from .density import density

def opportunity_gain(rcas, proximities, pci):
  
  # turn proximities in to ratios out of total
  prox_ratio = proximities / proximities.sum()
  
  # get the inverse of the RCAs matrix. Since they are in the form of 1's and
  # 0's this will flip all of them i.e. 1 = 0 and 0 = 1
  inverse_rcas = 1 - rcas
  
  # convert PCIs to a matrix of repeated values
  pci_matrix = pd.DataFrame(pci.reshape((1, len(pci))), columns=pci.index)
  pci_matrix = pci_matrix.reindex(index=rcas.index, method="ffill") # forward fill
  
  # here we now have the middle part of the equation
  middle = inverse_rcas * pci_matrix
  
  # get the density with the backwards bizzaro RCAs
  dcp = density(1-rcas, proximities)
  # now get the inverse
  dcp = 1-dcp
  # we now have the right-half of the equation
  right = dcp * pci_matrix
  
  # matrix multiplication with proximities ratio
  opp_gain = middle.dot(prox_ratio) - right
  
  return opp_gain